"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("../../converter/converter.module");
var di_1 = require("../../di");
var date_time_field_helper_service_1 = require("./date-time-field-helper.service");
var definitions_interface_1 = require("../../definitions.interface");
var dateTimeFieldHelper = di_1.staticInjector(date_time_field_helper_service_1.DateTimeFieldHelper);
describe('DateTimeFieldHelper', function () {
    describe('composeDateTimeRangeFields', function () {
        it('test1', function () {
            var value = dateTimeFieldHelper.composeDateTimeRangeFields({});
            expect(value).toEqual({
                fromDate: '2036-07-31T00:00:00-07:00',
                toDate: '2036-08-30T23:59:59-07:00',
            });
        });
        it('test2', function () {
            var value = dateTimeFieldHelper.composeDateTimeRangeFields({
                fromDate: '2017-01-01',
                toDate: '2017-01-31',
            });
            expect(value).toEqual({
                fromDate: '2017-01-01T00:00:00-08:00',
                toDate: '2017-01-31T23:59:59-08:00',
            });
        });
        it('test3', function () {
            var value = dateTimeFieldHelper.composeDateTimeRangeFields({
                fromDate: '2017-01-01',
            });
            expect(value).toEqual({
                fromDate: '2017-01-01T00:00:00-08:00',
                toDate: '2036-08-30T23:59:59-07:00',
            });
        });
        it('test4', function () {
            var value = dateTimeFieldHelper.composeDateTimeRangeFields({
                toDate: '2017-01-31',
            });
            expect(value).toEqual({
                fromDate: '2036-07-31T00:00:00-07:00',
                toDate: '2017-01-31T23:59:59-08:00',
            });
        });
        it('test5', function () {
            var value = dateTimeFieldHelper.composeDateTimeRangeFields({
                fromDate: '2017-01-01',
                fromTime: '10:34:59',
                toDate: '2017-01-31',
                toTime: '22:38:57',
            });
            expect(value).toEqual({
                fromDate: '2017-01-01T10:34:59-08:00',
                toDate: '2017-01-31T22:38:57-08:00',
            });
        });
        it('test6', function () {
            var value = dateTimeFieldHelper.composeDateTimeRangeFields({
                fromTime: '13:10:00',
                toTime: '22:38:57',
            });
            expect(value).toEqual({
                fromDate: '2036-07-31T13:10:00-07:00',
                toDate: '2036-08-30T22:38:57-07:00',
            });
        });
    });
    describe('composeDateTimeSinceField', function () {
        it('test1', function () {
            var value = dateTimeFieldHelper.composeDateTimeSinceField({
                changes: {
                    fromDate: '2017-01-01',
                    fromTime: '20:00:00',
                },
            });
            expect(value).toEqual({
                fromDate: '2017-01-01T20:00:00-08:00',
            });
        });
        it('test2', function () {
            var value = dateTimeFieldHelper.composeDateTimeSinceField({
                changes: {
                    fromDate: '2017-01-01',
                },
            });
            expect(value).toEqual({
                fromDate: '2017-01-01T00:00:00-08:00',
            });
        });
        it('test3', function () {
            var value = dateTimeFieldHelper.composeDateTimeSinceField({
                changes: {
                    fromDate: '2017-02-01',
                },
                entity: {
                    fromDate: '2017-01-01',
                },
            });
            expect(value).toEqual({
                fromDate: '2017-02-01T00:00:00-08:00',
            });
        });
        it('test4', function () {
            var value = dateTimeFieldHelper.composeDateTimeSinceField({
                changes: {
                    fromDate: null,
                },
                entity: {
                    fromDate: '2017-01-01',
                    fromTime: '10:59:59',
                },
            });
            expect(value).toEqual({ fromDate: null });
        });
        it('test5', function () {
            var value = dateTimeFieldHelper.composeDateTimeSinceField({
                changes: {},
                entity: {
                    fromDate: '2017-01-01',
                    fromTime: '10:59:59',
                },
            });
            expect(value).toEqual({ fromDate: definitions_interface_1.UNDEF });
        });
        it('test6', function () {
            var value = dateTimeFieldHelper.composeDateTimeSinceField({
                changes: {
                    fromTime: '09:49:49',
                },
                entity: {
                    fromDate: '2017-01-01',
                    fromTime: '10:59:59',
                },
            });
            expect(value).toEqual({ fromDate: '2017-01-01T09:49:49-08:00' });
        });
        it('test7', function () {
            var value = dateTimeFieldHelper.composeDateTimeSinceField({
                changes: {
                    fromTime: null,
                },
                entity: {
                    fromDate: '2017-01-01',
                    fromTime: '10:59:59',
                },
            });
            expect(value).toEqual({ fromDate: '2017-01-01T00:00:00-08:00' });
        });
    });
    describe('composeDateTimeTillField', function () {
        it('test1', function () {
            var value = dateTimeFieldHelper.composeDateTimeTillField({
                changes: {
                    toDate: '2017-01-01',
                    toTime: '20:00:00',
                },
            });
            expect(value).toEqual({
                toDate: '2017-01-01T20:00:00-08:00',
            });
        });
        it('test2', function () {
            var value = dateTimeFieldHelper.composeDateTimeTillField({
                changes: {
                    toDate: '2017-01-01',
                },
            });
            expect(value).toEqual({
                toDate: '2017-01-01T23:59:59-08:00',
            });
        });
    });
});
//# sourceMappingURL=date-time-field-helper.spec.js.map