"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var react_router_dom_1 = require("react-router-dom");
var MuiThemeProvider_1 = require("material-ui/styles/MuiThemeProvider");
var util_1 = require("../../util");
var di_1 = require("../../di");
var storage_1 = require("../../storage");
var notification_1 = require("../../notification");
var root_1 = require("../root");
var connector_1 = require("../connector");
var env_1 = require("../../env");
var transport_1 = require("../../transport");
var application_interface_1 = require("./application.interface");
var message_1 = require("../message");
var configurations_definitions_interface_1 = require("../../configurations-definitions.interface");
var universal_application_container_1 = require("./universal-application.container");
var channel_1 = require("../../channel");
var ApplicationContainer = /** @class */ (function (_super) {
    __extends(ApplicationContainer, _super);
    function ApplicationContainer(props) {
        var _this = _super.call(this, props) || this;
        _this.onUnload = _this.onUnload.bind(_this);
        return _this;
    }
    ApplicationContainer.prototype.render = function () {
        return (React.createElement(MuiThemeProvider_1.default, null,
            React.createElement(react_router_dom_1.BrowserRouter, { ref: 'router', basename: this.props.basename || env_1.ENV.basePath },
                React.createElement(react_router_dom_1.Switch, null, this.getRoutes()))));
    };
    /**
     * @stable [19.05.2018]
     */
    ApplicationContainer.prototype.componentDidMount = function () {
        this.eventManager.add(window, 'unload', this.onUnload);
        di_1.appContainer.bind(di_1.DI_TYPES.Router).toConstantValue(this.dynamicRouter);
        _super.prototype.componentDidMount.call(this);
    };
    /**
     * @stable [23.06.2018]
     */
    ApplicationContainer.prototype.componentWillUnmount = function () {
        this.eventManager.remove(window, 'unload', this.onUnload);
    };
    ApplicationContainer.prototype.onUnload = function () {
        if (this.settings.usePersistence) {
            this.saveState();
        }
    };
    ApplicationContainer.prototype.clearStateBeforeSerialization = function (state) {
        var predicates = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            predicates[_i - 1] = arguments[_i];
        }
        this.clearSystemState(state);
        // You may clear the app state here before the serializing
        if (predicates.length) {
            return util_1.cloneUsingFilters.apply(void 0, [state].concat(predicates));
        }
        return state;
    };
    /**
     * @stable [02.07.2018]
     * @param {TStoreEntity} state
     */
    ApplicationContainer.prototype.clearSystemState = function (state) {
        state.notification = notification_1.INITIAL_APPLICATION_NOTIFICATION_STATE;
        state.transport = transport_1.INITIAL_APPLICATION_TRANSPORT_STATE;
        state.application = application_interface_1.INITIAL_APPLICATION_STATE;
        state.channel = channel_1.INITIAL_APPLICATION_CHANNEL_STATE;
    };
    ApplicationContainer.prototype.getRoutes = function () {
        var props = this.props;
        return this.isMessageVisible()
            ? [
                React.createElement(message_1.Message, { key: util_1.uuid(), error: props.error, progress: props.progress, errorMessage: this.getErrorMessage(), emptyMessage: this.settings.messages.appNotReadyMessage })
            ]
            : _super.prototype.getRoutes.call(this);
    };
    ApplicationContainer.prototype.clearPreviousStates = function () {
        var _this = this;
        this.storage.each(function (o, key) {
            if (key.endsWith(storage_1.APPLICATION_STATE_KEY)) {
                _this.storage.remove(key, true);
            }
        });
    };
    ApplicationContainer.prototype.buildRoute = function (ctor, connectorConfiguration, cfg) {
        var Component;
        switch (cfg.type) {
            case configurations_definitions_interface_1.ContainerVisibilityTypeEnum.PRIVATE:
                Component = root_1.PrivateRootContainer;
                break;
            case configurations_definitions_interface_1.ContainerVisibilityTypeEnum.PUBLIC:
                Component = root_1.PublicRootContainer;
                break;
        }
        var props = __assign({ exact: true, accessConfiguration: connectorConfiguration.accessConfiguration, section: Reflect.get(ctor, connector_1.CONNECTOR_SECTION_FIELD) }, cfg);
        return (React.createElement(Component, __assign({ key: cfg.path || cfg.key, container: ctor }, props)));
    };
    ApplicationContainer.prototype.saveState = function () {
        this.clearPreviousStates();
        this.storage.set(storage_1.APPLICATION_STATE_KEY, this.clearStateBeforeSerialization(util_1.clone(this.appStore.getState())));
    };
    Object.defineProperty(ApplicationContainer.prototype, "dynamicRouter", {
        get: function () {
            // We cannot to get access to history instance other way. This instance is private
            return Reflect.get(this.refs.router, 'history');
        },
        enumerable: true,
        configurable: true
    });
    __decorate([
        di_1.lazyInject(di_1.DI_TYPES.Storage),
        __metadata("design:type", Object)
    ], ApplicationContainer.prototype, "storage", void 0);
    __decorate([
        di_1.lazyInject(di_1.DI_TYPES.EventManager),
        __metadata("design:type", Object)
    ], ApplicationContainer.prototype, "eventManager", void 0);
    return ApplicationContainer;
}(universal_application_container_1.UniversalApplicationContainer));
exports.ApplicationContainer = ApplicationContainer;
//# sourceMappingURL=application.container.js.map