"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var R = require("ramda");
var Printf = require("sprintf-js");
var ts_smart_logger_1 = require("ts-smart-logger");
var util_1 = require("../../../util");
var definitions_interface_1 = require("../../../definitions.interface");
var field_interface_1 = require("./field.interface");
var universal_component_1 = require("../../base/universal.component");
var field_support_1 = require("./field.support");
var UniversalField = /** @class */ (function (_super) {
    __extends(UniversalField, _super);
    /**
     * @stable [17.06.2018]
     * @param {TProps} props
     */
    function UniversalField(props) {
        var _this = _super.call(this, props) || this;
        _this.onClick = _this.onClick.bind(_this);
        _this.onChange = _this.onChange.bind(_this);
        _this.onChangeManually = _this.onChangeManually.bind(_this);
        _this.onFocus = _this.onFocus.bind(_this);
        _this.onBlur = _this.onBlur.bind(_this);
        _this.onKeyUp = _this.onKeyUp.bind(_this);
        _this.onKeyDown = _this.onKeyDown.bind(_this);
        _this.closeSyntheticKeyboard = _this.closeSyntheticKeyboard.bind(_this);
        _this.state = {};
        if (props.useKeyboard) {
            _this.caretBlinkingTask = new util_1.DelayedTask(_this.setCaretVisibility.bind(_this), props.caretBlinkingFrequencyTimeout || UniversalField.DEFAULT_CARET_BLINKING_FREQUENCY_TIMEOUT, true);
        }
        return _this;
    }
    /**
     * @stable [04.09.2018]
     */
    UniversalField.prototype.componentWillUnmount = function () {
        _super.prototype.componentWillUnmount.call(this);
        this.onCloseKeyboard();
    };
    /**
     * @stable [20.08.2018]
     */
    UniversalField.prototype.componentDidMount = function () {
        _super.prototype.componentDidMount.call(this);
        // Need to invoke a user validator if it exists (After F5, etc...)
        this.validateValueAndSetCustomValidity(this.value);
        if (this.props.autoFocus) {
            this.setFocus(); // The manual "autoFocus" replacing
        }
    };
    /**
     * @stable [04.09.2018]
     * @param {TProps} prevProps
     * @param {TState} prevState
     */
    UniversalField.prototype.componentDidUpdate = function (prevProps, prevState) {
        _super.prototype.componentDidUpdate.call(this, prevProps, prevState);
        if (this.props.useKeyboard
            && this.state.keyboardOpened
            && !R.equals(this.value, prevProps.value)) {
            // Refresh a caret position according to a changed value
            this.refreshCaretPosition();
        }
    };
    /**
     * @stable [17.06.2018]
     */
    UniversalField.prototype.resetError = function () {
        this.validateField(definitions_interface_1.CLEAR_DIRTY_CHANGES_VALUE);
    };
    /**
     * @stable [20.08.2018]
     */
    UniversalField.prototype.clearValue = function () {
        this.setFocus(); // UX
        if (this.isValuePresent()) {
            this.onChangeManually(this.getEmptyValue());
        }
        var props = this.props;
        if (util_1.isFn(props.onClear)) {
            props.onClear();
        }
    };
    /**
     * @stable [27.07.2018]
     * @param {AnyT} event
     */
    UniversalField.prototype.onChange = function (event) {
        this.onChangeValue(this.getRawValueFromEvent(event));
    };
    /**
     * @stable [17.06.2018]
     * @param {AnyT} currentRawValue
     * @param {AnyT} context
     */
    UniversalField.prototype.onChangeManually = function (currentRawValue, context) {
        this.onChangeValue(currentRawValue);
    };
    Object.defineProperty(UniversalField.prototype, "value", {
        /**
         * @stable [22.10.2018]
         * @returns {AnyT}
         */
        get: function () {
            var props = this.props;
            var value = props.value;
            return util_1.isDef(value) ? value : props.defaultValue;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [18.06.2018]
     * @param {TKeyboardEvent} event
     */
    UniversalField.prototype.onKeyBackspace = function (event) {
        var props = this.props;
        if (util_1.isFn(props.onKeyBackspace)) {
            props.onKeyBackspace(event);
        }
    };
    /**
     * @stable [18.06.2018]
     * @param {TKeyboardEvent} event
     */
    UniversalField.prototype.onKeyUp = function (event) {
        var props = this.props;
        if (util_1.isFn(props.onKeyUp)) {
            props.onKeyUp(event);
        }
    };
    /**
     * @stable [03.09.2018]
     * @param {TKeyboardEvent} event
     */
    UniversalField.prototype.onKeyDown = function (event) {
        var props = this.props;
        if (util_1.isFn(props.onKeyDown)) {
            props.onKeyDown(event);
        }
    };
    /**
     * @stable [18.06.2018]
     * @param {TKeyboardEvent} event
     */
    UniversalField.prototype.onKeyEnter = function (event) {
        var props = this.props;
        if (util_1.isFn(props.onKeyEnter)) {
            props.onKeyEnter(event);
        }
    };
    /**
     * @stable [04.09.2018]
     * @param {TKeyboardEvent} event
     */
    UniversalField.prototype.onKeyTab = function (event) {
        var props = this.props;
        if (util_1.isFn(props.onKeyTab)) {
            props.onKeyTab(event);
        }
    };
    /**
     * @stable [18.06.2018]
     * @param {TKeyboardEvent} event
     */
    UniversalField.prototype.onKeyEscape = function (event) {
        var props = this.props;
        if (util_1.isFn(props.onKeyEscape)) {
            props.onKeyEscape(event);
        }
    };
    /**
     * @stable [18.06.2018]
     * @param {TKeyboardEvent} event
     */
    UniversalField.prototype.onKeyArrowDown = function (event) {
        var props = this.props;
        if (util_1.isFn(props.onKeyArrowDown)) {
            props.onKeyArrowDown(event);
        }
    };
    /**
     * @stable [18.06.2018]
     * @param {TKeyboardEvent} event
     */
    UniversalField.prototype.onKeyArrowUp = function (event) {
        var props = this.props;
        if (util_1.isFn(props.onKeyArrowUp)) {
            props.onKeyArrowUp(event);
        }
    };
    /**
     * @stable [18.06.2018]
     * @param {boolean} usePrintf
     * @param {AnyT} args
     * @returns {string}
     */
    UniversalField.prototype.printfDisplayMessage = function (usePrintf) {
        var _this = this;
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        return util_1.orDefault(usePrintf, function () { return Printf.sprintf.apply(Printf, [_this.t(_this.props.displayMessage)].concat(args)); }, field_interface_1.FIELD_DISPLAY_EMPTY_VALUE);
    };
    /**
     * The state may be an external storage and the value must be able to be serialized.
     *
     * @stable [17.06.2018]
     * @param {AnyT} rawValue
     * @returns {AnyT}
     */
    UniversalField.prototype.toSerializedValue = function (rawValue) {
        return rawValue;
    };
    /**
     * @stable [17.06.2018]
     * @param {AnyT} value
     * @returns {boolean}
     */
    UniversalField.prototype.isValuePresent = function (value) {
        if (value === void 0) { value = this.value; }
        return util_1.isDef(value) && !R.equals(value, this.getEmptyValue());
    };
    /**
     * @stable [18.12.2018]
     * @returns {boolean}
     */
    UniversalField.prototype.isNotDefaultValuePresent = function () {
        var props = this.props;
        return util_1.isDef(props.value) && util_1.isDef(props.defaultValue) && !R.equals(props.value, props.defaultValue);
    };
    /**
     * @stable [05.10.2018]
     * @returns {boolean}
     */
    UniversalField.prototype.isFieldRequired = function () {
        return field_support_1.isFieldRequired(this.props);
    };
    /**
     * @stable [05.10.2018]
     * @returns {boolean}
     */
    UniversalField.prototype.isFieldInvalid = function () {
        return !R.isNil(this.error);
    };
    /**
     * @stable [17.06.2018]
     * @returns {AnyT}
     */
    UniversalField.prototype.getEmptyValue = function () {
        return field_interface_1.FIELD_DISPLAY_EMPTY_VALUE;
    };
    Object.defineProperty(UniversalField.prototype, "definiteValue", {
        /**
         * @stable [18.06.2018]
         * @returns {AnyT}
         */
        get: function () {
            return util_1.isUndef(this.value) ? this.getEmptyValue() : this.value;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [31.07.2018]
     * @param {AnyT} value
     * @returns {string}
     */
    UniversalField.prototype.validateValueAndSetCustomValidity = function (value) {
        var props = this.props;
        var error = field_interface_1.FIELD_EMPTY_ERROR_VALUE;
        this.setNativeInputValidity('');
        if (this.isInputValid()) {
            error = util_1.isFn(props.validate) ? props.validate(value) : error;
            if (R.isNil(error)) {
                error = this.validateValue(value); // The custom internal validator
            }
            if (!R.isNil(error)) {
                this.setNativeInputValidity(error);
            }
        }
        else {
            error = this.getNativeInputValidationMessage();
        }
        return error;
    };
    /**
     * @stable [31.07.2018]
     * @param {AnyT} value
     * @returns {string}
     */
    UniversalField.prototype.validateValue = function (value) {
        return field_interface_1.FIELD_EMPTY_ERROR_VALUE;
    };
    /**
     * @stable [03.09.2018]
     * @returns {boolean}
     */
    UniversalField.prototype.isInputValid = function () {
        return true;
    };
    /**
     * @stable [31.07.2018]
     * @returns {string}
     */
    UniversalField.prototype.getNativeInputValidationMessage = function () {
        return field_interface_1.FIELD_EMPTY_ERROR_VALUE;
    };
    /**
     * @stable [31.07.2018]
     * @param {string} error
     */
    UniversalField.prototype.setNativeInputValidity = function (error) {
        // Do nothing
    };
    /**
     * @stable [17.06.2018]
     * @returns {Array<string | RegExp>}
     */
    UniversalField.prototype.getFieldMask = function () {
        return this.props.mask;
    };
    /**
     * @stable [17.06.2018]
     * @returns {string}
     */
    UniversalField.prototype.getFieldPattern = function () {
        return this.props.pattern;
    };
    Object.defineProperty(UniversalField.prototype, "displayValue", {
        /**
         * @stable [18.06.2018]
         * @returns {AnyT}
         */
        get: function () {
            return this.toDisplayValue(this.value);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [18.06.2018]
     * @param {AnyT} value
     * @param {AnyT} context
     * @returns {AnyT}
     */
    UniversalField.prototype.toDisplayValue = function (value, context) {
        var props = this.props;
        var displayValue = props.displayValue;
        return this.inProgress()
            ? field_interface_1.FIELD_DISPLAY_EMPTY_VALUE // The dictionaries data is cleaned before request
            : (this.isValuePresent(value)
                ? (util_1.isUndef(displayValue)
                    ? value
                    : (util_1.isFn(displayValue)
                        ? displayValue(value, this)
                        // TODO remove scope and use tryCalcDisplayValue
                        : displayValue))
                : field_interface_1.FIELD_DISPLAY_EMPTY_VALUE);
    };
    /**
     * @stable [03.12.2018]
     * @param {AnyT} value
     * @returns {AnyT}
     */
    UniversalField.prototype.tryCalcDisplayValue = function (value) {
        var props = this.props;
        var displayValue = props.displayValue;
        return util_1.isFn(displayValue) ? util_1.calc(displayValue, value) : value;
    };
    /**
     * @stable [06.10.2018]
     * @returns {boolean}
     */
    UniversalField.prototype.isFieldInactive = function () {
        return field_support_1.isFieldInactive(this.props) || this.inProgress();
    };
    /**
     * @stable [06.10.2018]
     * @returns {boolean}
     */
    UniversalField.prototype.isFieldDeactivated = function () {
        return this.props.disabled || this.inProgress();
    };
    /**
     * @stable [06.10.2018]
     * @returns {boolean}
     */
    UniversalField.prototype.isFieldChangeable = function () {
        return field_support_1.isFieldChangeable(this.props);
    };
    /**
     * @stable [06.10.2018]
     * @returns {boolean}
     */
    UniversalField.prototype.inProgress = function () {
        return field_support_1.isFieldInProgress(this.props);
    };
    /**
     * @stable [04.09.2018]
     * @param {TFocusEvent} event
     */
    UniversalField.prototype.onFocus = function (event) {
        var props = this.props;
        if (props.preventFocus || props.useKeyboard) {
            this.removeFocus(); // Prevent native keyboard opening when using a synthetic keyboard
            this.openSyntheticKeyboard(); // "useKeyboard" props checking inside
        }
        if (util_1.isFn(props.onFocus)) {
            props.onFocus(event);
        }
    };
    /**
     * @stable [18.06.2018]
     * @param {TFocusEvent} event
     */
    UniversalField.prototype.onBlur = function (event) {
        var props = this.props;
        if (util_1.isFn(props.onBlur)) {
            props.onBlur(event);
        }
    };
    /**
     * @stable [18.06.2018]
     * @param {TBasicEvent} event
     */
    UniversalField.prototype.onClick = function (event) {
        var props = this.props;
        if (util_1.isFn(props.onClick)) {
            props.onClick(event);
        }
    };
    /**
     * @stable [03.09.2018]
     * @returns {JSX.Element}
     */
    UniversalField.prototype.getInputAttachmentElement = function () {
        return null;
    };
    /**
     * @stable [03.09.2018]
     * @returns {JSX.Element}
     */
    UniversalField.prototype.toKeyboardElement = function () {
        return null;
    };
    /**
     * @stable [04.09.2018]
     */
    UniversalField.prototype.openSyntheticKeyboard = function () {
        var _this = this;
        if (!this.props.useKeyboard || this.state.keyboardOpened) {
            return false;
        }
        this.setState({ keyboardOpened: true }, function () { return _this.caretBlinkingTask.start(); });
        UniversalField.logger.debug("[$UniversalField][openSyntheticKeyboard] A keyboard for the field \"" + this.props.name + "\" will be opened soon.");
        return true;
    };
    /**
     * @stable [04.09.2018]
     */
    UniversalField.prototype.closeSyntheticKeyboard = function () {
        if (!this.props.useKeyboard) {
            return;
        }
        this.setState({ keyboardOpened: false });
        this.onCloseKeyboard();
    };
    /**
     * @stable [04.09.2018]
     */
    UniversalField.prototype.onCloseKeyboard = function () {
        if (this.props.useKeyboard) {
            this.caretBlinkingTask.stop();
            UniversalField.logger.debug("[$UniversalField][onCloseKeyboard] A keyboard for the field \"" + this.props.name + "\" has been closed.");
            return true;
        }
        return false;
    };
    /**
     * @stable [03.09.2018]
     * @returns {JSX.Element}
     */
    UniversalField.prototype.getKeyboardElement = function () {
        var _this = this;
        return (util_1.orNull(this.props.useKeyboard && this.state.keyboardOpened, function () { return _this.toKeyboardElement(); }));
    };
    /**
     * @stable [03.09.2018]
     * @returns {JSX.Element}
     */
    UniversalField.prototype.getErrorMessageElement = function () {
        var _this = this;
        return util_1.orNull(this.props.errorMessageRendered !== false, function () { return _this.toMessageElement(_this.error, _this.uiFactory.fieldValidationText); });
    };
    /**
     * @stable [03.09.2018]
     * @returns {JSX.Element}
     */
    UniversalField.prototype.getMessageElement = function () {
        var _this = this;
        var props = this.props;
        var message = props.message;
        return util_1.orNull(message, function () { return _this.toMessageElement(message); });
    };
    Object.defineProperty(UniversalField.prototype, "error", {
        /**
         * @stable [03.09.2018]
         * @returns {string}
         */
        get: function () {
            return this.state.error;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [04.09.2018]
     */
    UniversalField.prototype.refreshCaretPosition = function () {
        this.setState({ caretPosition: this.getCaretPosition() });
        UniversalField.logger.debug("[$UniversalField][refreshCaretPosition] A caret position for the field " + this.props.name + " had been refreshed.\".");
    };
    /**
     * @stable [04.09.2018]
     * @returns {number}
     */
    UniversalField.prototype.getCaretPosition = function () {
        // Need to implement
        return 0;
    };
    /**
     * @stable [31.07.2018]
     * @param {AnyT} currentRawValue
     */
    UniversalField.prototype.onChangeValue = function (currentRawValue) {
        var props = this.props;
        var actualChangedValue = field_support_1.toActualChangedValue({
            value: currentRawValue,
            emptyValue: this.getEmptyValue(),
            originalValue: props.originalValue,
            canReturnClearDirtyChangesValue: props.canReturnClearDirtyChangesValue,
        });
        this.validateField(actualChangedValue);
        this.propsOnChange(actualChangedValue);
        this.propsChangeForm(actualChangedValue); // Notify the form about changes
    };
    /**
     * @stable [31.07.2018]
     * @param {AnyT} rawValue
     */
    UniversalField.prototype.validateField = function (rawValue) {
        // State value cannot take an undefined value then we should pass a null value at least
        this.setState({ error: this.validateValueAndSetCustomValidity(rawValue) });
    };
    /**
     * @stable [17.06.2018]
     * @param {AnyT} rawValue
     */
    UniversalField.prototype.propsChangeForm = function (rawValue) {
        var props = this.props;
        if (!util_1.isFn(props.changeForm)) {
            return;
        }
        props.changeForm(props.name, this.toSerializedValue(rawValue), props.validationGroup);
    };
    /**
     * @stable [17.06.2018]
     * @param {AnyT} rawValue
     */
    UniversalField.prototype.propsOnChange = function (rawValue) {
        var props = this.props;
        if (props.onChange) {
            props.onChange(props.returnSerializedValueOnChange ? this.toSerializedValue(rawValue) : rawValue);
        }
    };
    /**
     * @stable [04.09.2018]
     */
    UniversalField.prototype.setCaretVisibility = function () {
        var _this = this;
        this.setState(util_1.defValuesFilter({
            caretVisibility: !this.state.caretVisibility,
            // Lazy initialization to do not use a setTimeout
            caretPosition: util_1.orUndef(R.isNil(this.state.caretPosition), function () { return _this.getCaretPosition(); }),
        }));
    };
    UniversalField.logger = ts_smart_logger_1.LoggerFactory.makeLogger('UniversalField');
    UniversalField.DEFAULT_CARET_BLINKING_FREQUENCY_TIMEOUT = 400;
    return UniversalField;
}(universal_component_1.UniversalComponent));
exports.UniversalField = UniversalField;
//# sourceMappingURL=universal-field.component.js.map