import { IEntity, IKeyValue, AnyT } from '../../../definitions.interface';
import { IMultiItemEntity } from '../../../entities-definitions.interface';
import { VueField } from '../field/vue-index';
import { MultiFieldPlugin } from '../multifield/vue-index';
import { IVueBaseFileViewerProps, IVueViewerListenersEntity } from '../../viewer/vue-index';
import { IVueBaseFileFieldTemplateMethods } from './vue-filefield.interface';
export declare class VueBaseFileField extends VueField implements IVueBaseFileFieldTemplateMethods {
    protected readonly maxFiles: number;
    protected readonly type: string;
    protected readonly viewer: string;
    protected readonly emptyMessageFactory: (index: number) => string;
    protected readonly placeHolderMessageFactory: (index: number) => string;
    protected readonly clsFactory: (entity: IEntity) => string;
    protected readonly subFieldConfig: (index: number) => IKeyValue;
    protected readonly viewerProps: IVueBaseFileViewerProps;
    protected readonly displayFileName: string;
    protected readonly displayFileFormat: string;
    protected readonly multiFieldPlugin: MultiFieldPlugin;
    protected readonly cachedFiles: Map<string, File>;
    /**
     * @stable [27.11.2018]
     */
    constructor();
    /**
     * @stable [25.11.2018]
     * @returns {string}
     */
    getFieldClassName(): string;
    /**
     * @stable [16.12.2018]
     * @param {File[]} files
     * @param {number} index
     * @returns {string[]}
     */
    onFilesSelect(files: File[], index: number): string[];
    /**
     * @stable [09.12.2018]
     * @param {number} index
     * @returns {string}
     */
    getLabel(index: number): string;
    getPlaceholder(index: number): string;
    /**
     * @stable [20.12.2018]
     * @returns {IEntity[]}
     */
    getFiles(): IEntity[];
    /**
     * @stable [09.12.2018]
     * @param {IMultiItemEntity} entity
     * @param {number} index
     * @returns {IVueBaseFileViewerProps}
     */
    getViewerBindings(entity: IMultiItemEntity | string, index: number): IVueBaseFileViewerProps;
    /**
     * @stable [16.12.2018]
     * @param {IMultiItemEntity} fileEntity
     * @param {number} index
     * @returns {IVueViewerListenersEntity<File>}
     */
    getViewerListeners(fileEntity: IMultiItemEntity, index: number): IVueViewerListenersEntity<File>;
    /**
     * @stable [21.12.2018]
     * @param {File} file
     * @param {number} index
     * @returns {string}
     */
    getViewerComponent(file: File, index: number): string;
    /**
     * @stable [17.11.2018]
     * @returns {string}
     */
    protected getInputClassName(): string;
    /**
     * @stable [09.12.2018]
     * @returns {string}
     */
    protected getFieldAttachmentTemplate(): string;
    /**
     * @stable [16.12.2018]
     * @param {IMultiItemEntity} fileEntity
     * @param {File} file
     * @param {number} index
     */
    protected onFileChange(fileEntity: IMultiItemEntity, file: File, index: number): void;
    /**
     * @stable [10.12.2018]
     * @returns {TMethods}
     */
    protected getTemplateMethods<TMethods extends IVueBaseFileFieldTemplateMethods>(): TMethods;
    protected getFileFormat(): AnyT;
    /**
     * @stable [10.12.2018]
     * @param {File} file
     * @returns {string}
     */
    private toFileUrl;
    /**
     * @stable [28.11.2018]
     * @param {string} fileUrl
     */
    private destroyFileUrl;
    /**
     * @stable [16.12.2018]
     * @param {string} fileUrl
     * @param {number} index
     */
    private addFile;
    /**
     * @stable [28.11.2018]
     * @param {IMultiItemEntity} fileEntity
     */
    private removeFile;
    /**
     * @stable [16.12.2018]
     * @param {number} index
     * @returns {IKeyValue}
     */
    private getSubFieldConfig;
    private getFileName;
}
