"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var Promise = require("bluebird");
var React = require("react");
var R = require("ramda");
var base_1 = require("../../base");
var menu_1 = require("../../menu");
var util_1 = require("../../../util");
var layout_1 = require("../../layout");
var GoogleMaps = /** @class */ (function (_super) {
    __extends(GoogleMaps, _super);
    /**
     * @stable [31.07.2018]
     * @param {IGoogleMapsProps} props
     */
    function GoogleMaps(props) {
        var _this = _super.call(this, props) || this;
        _this.markers = new Map();
        _this.markersDragListeners = new Map();
        _this.onMapClick = _this.onMapClick.bind(_this);
        _this.onMenuSelect = _this.onMenuSelect.bind(_this);
        _this.openMenu = _this.openMenu.bind(_this);
        _this.initGoogleMapsObjects = _this.initGoogleMapsObjects.bind(_this);
        if (_this.hasOptions) {
            _this.delayedTask = new util_1.DelayedTask(_this.openMenu, 200);
        }
        return _this;
    }
    /**
     * @stable [31.07.2018]
     */
    GoogleMaps.prototype.componentDidMount = function () {
        _super.prototype.componentDidMount.call(this);
        // We cannot cancel the original promise, because of it is shared
        this.googleMapsScriptTask = new Promise(function (resolve, reject) { return util_1.getGoogleMapsScript().then(resolve, reject); })
            .then(this.initGoogleMapsObjects);
    };
    /**
     * @stable [31.07.2018]
     */
    GoogleMaps.prototype.componentWillUnmount = function () {
        _super.prototype.componentWillUnmount.call(this);
        if (util_1.isDef(this.delayedTask)) {
            this.delayedTask.stop();
        }
        this.cancelGoogleMapsScriptTaskIfPending();
        this.markersDragListeners.forEach(function (dragEndEventListener) { return dragEndEventListener.remove(); });
        if (util_1.isDef(this.clickEventListener)) {
            this.clickEventListener.remove();
        }
        if (util_1.isDef(this.dbClickEventListener)) {
            this.dbClickEventListener.remove();
        }
        this.map = null;
        this.markers = null;
        this.delayedTask = null;
        this.googleMapsScriptTask = null;
        this.markersDragListeners = null;
    };
    /**
     * @stable [31.07.2018]
     * @returns {JSX.Element}
     */
    GoogleMaps.prototype.render = function () {
        var _this = this;
        var props = this.props;
        return (React.createElement(layout_1.FlexLayout, { className: 'rac-google-maps' },
            React.createElement("div", { ref: this.getSelfRef(), className: 'rac-google-maps-map rac-flex-full' }),
            util_1.orNull(this.hasOptions, function () { return (React.createElement(menu_1.Menu, { ref: 'menu', renderToX: function () { return _this.x + _this.getSelf().offsetLeft; }, renderToY: function () { return _this.y + _this.getSelf().offsetTop; }, options: props.options, onSelect: _this.onMenuSelect })); })));
    };
    /**
     * @stable [31.07.2018]
     * @param {google.maps.MarkerOptions} cfg
     * @param {string} name
     * @returns {google.maps.Marker}
     */
    GoogleMaps.prototype.addMarker = function (cfg, name) {
        var _this = this;
        var config = __assign({ anchorPoint: new google.maps.Point(0, -29), position: null }, cfg, { map: this.map });
        var markerId = name || util_1.uuid();
        var marker = new google.maps.Marker(config);
        this.markers.set(markerId, marker);
        if (config.draggable) {
            this.markersDragListeners.set(markerId, google.maps.event.addListener(marker, 'dragend', function () { return _this.onMarkerDragEnd(markerId, marker); }));
        }
        return marker;
    };
    /**
     * @stable [31.07.2018]
     * @param {string | google.maps.Marker} marker
     * @param {boolean} markerVisibility
     * @param {boolean} refreshMap
     * @param {number | undefined} lat
     * @param {number | undefined} lng
     * @param {number | undefined} zoom
     */
    GoogleMaps.prototype.setMarkerState = function (marker, markerVisibility, refreshMap, lat, lng, zoom) {
        if (lat === void 0) { lat = this.settings.googleMaps.lat; }
        if (lng === void 0) { lng = this.settings.googleMaps.lng; }
        if (zoom === void 0) { zoom = this.settings.googleMaps.zoom; }
        var markerAsObject = util_1.isString(marker) ? this.markers.get(marker) : marker;
        markerAsObject.setPosition({ lat: lat, lng: lng });
        markerAsObject.setVisible(markerVisibility);
        if (refreshMap) {
            this.map.setCenter({ lat: lat, lng: lng });
            this.map.setZoom(zoom);
        }
    };
    /**
     * @stable [31.07.2018]
     * @param {string} markerId
     * @param {google.maps.Marker} marker
     */
    GoogleMaps.prototype.onMarkerDragEnd = function (markerId, marker) {
        var position = marker.getPosition();
        var lat = position.lat();
        var lng = position.lng();
        var props = this.props;
        if (props.onChangePlace) {
            props.onChangePlace({ name: markerId, item: marker, lat: lat, lng: lng });
        }
    };
    /**
     * @stable [31.07.2018]
     * @param {IMenuItemEntity} item
     */
    GoogleMaps.prototype.onMenuSelect = function (item) {
        var props = this.props;
        if (props.onSelect) {
            props.onSelect({ item: item, lat: this.lat, lng: this.lng });
        }
    };
    /**
     * @stable [31.07.2018]
     * @param {{pixel: google.maps.Point; latLng: google.maps.LatLng}} event
     */
    GoogleMaps.prototype.onMapClick = function (event) {
        if (event.pixel) {
            var x = event.pixel.x;
            var y = event.pixel.y;
            var lat = event.latLng.lat();
            var lng = event.latLng.lng();
            this.delayedTask.start({ x: x, y: y, lat: lat, lng: lng });
        }
    };
    /**
     * @stable [31.07.2018]
     */
    GoogleMaps.prototype.cancelGoogleMapsScriptTaskIfPending = function () {
        this.cancelTaskIfPending(this.googleMapsScriptTask);
    };
    /**
     * @stable [31.07.2018]
     */
    GoogleMaps.prototype.cancelTaskIfPending = function (promise) {
        if (promise && promise.isPending()) {
            promise.cancel();
        }
    };
    /**
     * @stable [31.07.2018]
     * @param {{x: number; y: number; lat: number; lng: number}} nextState
     */
    GoogleMaps.prototype.openMenu = function (nextState) {
        Object.assign(this, nextState); // We need to sync the internal state with a starting task state
        this.menu.show();
    };
    /**
     * @stable [31.07.2018]
     */
    GoogleMaps.prototype.initGoogleMapsObjects = function () {
        var _this = this;
        this.map = new google.maps.Map(this.getSelf());
        if (this.hasOptions) {
            this.clickEventListener = google.maps.event.addListener(this.map, 'click', this.onMapClick);
            this.dbClickEventListener = google.maps.event.addListener(this.map, 'dblclick', function () { return _this.delayedTask.stop(); });
        }
        var props = this.props;
        if (props.onInit) {
            props.onInit(this.map);
        }
    };
    Object.defineProperty(GoogleMaps.prototype, "menu", {
        /**
         * @stable [31.07.2018]
         * @returns {IMenu}
         */
        get: function () {
            return this.refs.menu;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(GoogleMaps.prototype, "hasOptions", {
        /**
         * @stable [26.10.2018]
         * @returns {boolean}
         */
        get: function () {
            return !R.isNil(this.props.options);
        },
        enumerable: true,
        configurable: true
    });
    return GoogleMaps;
}(base_1.BaseComponent));
exports.GoogleMaps = GoogleMaps;
//# sourceMappingURL=google-maps.component.js.map